@@ -1325,17 +1325,39 @@ dumpCreateDB(PGconn *conn)
 				appendStringLiteralConn(buf, dbname, conn);
 				appendPQExpBuffer(buf, ";\n");
 			}
+		}
+		else if (strcmp(dbtablespace, "pg_default") != 0 && !no_tablespaces)
+		{
+			/*
+			 * Cannot change tablespace of the database we're connected to,
+			 * so to move "postgres" to another tablespace, we connect to
+			 * "template1", and vice versa.
+			 */
+			if (strcmp(dbname, "postgres") == 0)
+				appendPQExpBuffer(buf, "%s\\connect template1\n",
+					/* Add a space before \\connect so pg_upgrade can split */
+					binary_upgrade ? " " : "");
+			else
+				appendPQExpBuffer(buf, "%s\\connect postgres\n",
+					binary_upgrade ? " " : "");
 
-			if (binary_upgrade)
-			{
-				appendPQExpBuffer(buf, "-- For binary upgrade, set datfrozenxid.\n");
-				appendPQExpBuffer(buf, "UPDATE pg_catalog.pg_database "
-								  "SET datfrozenxid = '%u' "
-								  "WHERE datname = ",
-								  dbfrozenxid);
-				appendStringLiteralConn(buf, dbname, conn);
-				appendPQExpBuffer(buf, ";\n");
-			}
+			appendPQExpBuffer(buf, "ALTER DATABASE %s SET TABLESPACE %s;\n",
+							  fdbname, fmtId(dbtablespace));
+
+			/* connect to original database */
+			appendPQExpBuffer(buf, "%s\\connect %s\n",
+				binary_upgrade ? " " : "", fdbname);
+		}
+
+		if (binary_upgrade)
+		{
+			appendPQExpBuffer(buf, "-- For binary upgrade, set datfrozenxid.\n");
+			appendPQExpBuffer(buf, "UPDATE pg_catalog.pg_database "
+							  "SET datfrozenxid = '%u' "
+							  "WHERE datname = ",
+							  dbfrozenxid);
+			appendStringLiteralConn(buf, dbname, conn);
+			appendPQExpBuffer(buf, ";\n");
 		}
 
 		if (!skip_acls &&
