@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 3.0.2.  */
+/* A Bison parser, made by GNU Bison 2.5.  */
 
 /* Bison interface for Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
-
+   
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
+   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,128 +26,122 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_PLPGSQL_YY_PL_GRAM_H_INCLUDED
-# define YY_PLPGSQL_YY_PL_GRAM_H_INCLUDED
-/* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int plpgsql_yydebug;
-#endif
 
-/* Token type.  */
+/* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-  enum yytokentype
-  {
-    IDENT = 258,
-    FCONST = 259,
-    SCONST = 260,
-    BCONST = 261,
-    XCONST = 262,
-    Op = 263,
-    ICONST = 264,
-    PARAM = 265,
-    TYPECAST = 266,
-    DOT_DOT = 267,
-    COLON_EQUALS = 268,
-    T_WORD = 269,
-    T_CWORD = 270,
-    T_DATUM = 271,
-    LESS_LESS = 272,
-    GREATER_GREATER = 273,
-    K_ABSOLUTE = 274,
-    K_ALIAS = 275,
-    K_ALL = 276,
-    K_ARRAY = 277,
-    K_BACKWARD = 278,
-    K_BEGIN = 279,
-    K_BY = 280,
-    K_CASE = 281,
-    K_CLOSE = 282,
-    K_COLLATE = 283,
-    K_CONSTANT = 284,
-    K_CONTINUE = 285,
-    K_CURSOR = 286,
-    K_DEBUG = 287,
-    K_DECLARE = 288,
-    K_DEFAULT = 289,
-    K_DETAIL = 290,
-    K_DIAGNOSTICS = 291,
-    K_DUMP = 292,
-    K_ELSE = 293,
-    K_ELSIF = 294,
-    K_END = 295,
-    K_ERRCODE = 296,
-    K_ERROR = 297,
-    K_EXCEPTION = 298,
-    K_EXECUTE = 299,
-    K_EXIT = 300,
-    K_FETCH = 301,
-    K_FIRST = 302,
-    K_FOR = 303,
-    K_FOREACH = 304,
-    K_FORWARD = 305,
-    K_FROM = 306,
-    K_GET = 307,
-    K_HINT = 308,
-    K_IF = 309,
-    K_IN = 310,
-    K_INFO = 311,
-    K_INSERT = 312,
-    K_INTO = 313,
-    K_IS = 314,
-    K_LAST = 315,
-    K_LOG = 316,
-    K_LOOP = 317,
-    K_MESSAGE = 318,
-    K_MOVE = 319,
-    K_NEXT = 320,
-    K_NO = 321,
-    K_NOT = 322,
-    K_NOTICE = 323,
-    K_NULL = 324,
-    K_OPEN = 325,
-    K_OPTION = 326,
-    K_OR = 327,
-    K_PERFORM = 328,
-    K_PRIOR = 329,
-    K_QUERY = 330,
-    K_RAISE = 331,
-    K_RELATIVE = 332,
-    K_RESULT_OID = 333,
-    K_RETURN = 334,
-    K_REVERSE = 335,
-    K_ROWTYPE = 336,
-    K_ROW_COUNT = 337,
-    K_SCROLL = 338,
-    K_SLICE = 339,
-    K_SQLSTATE = 340,
-    K_STRICT = 341,
-    K_THEN = 342,
-    K_TO = 343,
-    K_TYPE = 344,
-    K_USE_COLUMN = 345,
-    K_USE_VARIABLE = 346,
-    K_USING = 347,
-    K_VARIABLE_CONFLICT = 348,
-    K_WARNING = 349,
-    K_WHEN = 350,
-    K_WHILE = 351
-  };
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     IDENT = 258,
+     FCONST = 259,
+     SCONST = 260,
+     BCONST = 261,
+     XCONST = 262,
+     Op = 263,
+     ICONST = 264,
+     PARAM = 265,
+     TYPECAST = 266,
+     DOT_DOT = 267,
+     COLON_EQUALS = 268,
+     T_WORD = 269,
+     T_CWORD = 270,
+     T_DATUM = 271,
+     LESS_LESS = 272,
+     GREATER_GREATER = 273,
+     K_ABSOLUTE = 274,
+     K_ALIAS = 275,
+     K_ALL = 276,
+     K_ARRAY = 277,
+     K_BACKWARD = 278,
+     K_BEGIN = 279,
+     K_BY = 280,
+     K_CASE = 281,
+     K_CLOSE = 282,
+     K_COLLATE = 283,
+     K_CONSTANT = 284,
+     K_CONTINUE = 285,
+     K_CURSOR = 286,
+     K_DEBUG = 287,
+     K_DECLARE = 288,
+     K_DEFAULT = 289,
+     K_DETAIL = 290,
+     K_DIAGNOSTICS = 291,
+     K_DUMP = 292,
+     K_ELSE = 293,
+     K_ELSIF = 294,
+     K_END = 295,
+     K_ERRCODE = 296,
+     K_ERROR = 297,
+     K_EXCEPTION = 298,
+     K_EXECUTE = 299,
+     K_EXIT = 300,
+     K_FETCH = 301,
+     K_FIRST = 302,
+     K_FOR = 303,
+     K_FOREACH = 304,
+     K_FORWARD = 305,
+     K_FROM = 306,
+     K_GET = 307,
+     K_HINT = 308,
+     K_IF = 309,
+     K_IN = 310,
+     K_INFO = 311,
+     K_INSERT = 312,
+     K_INTO = 313,
+     K_IS = 314,
+     K_LAST = 315,
+     K_LOG = 316,
+     K_LOOP = 317,
+     K_MESSAGE = 318,
+     K_MOVE = 319,
+     K_NEXT = 320,
+     K_NO = 321,
+     K_NOT = 322,
+     K_NOTICE = 323,
+     K_NULL = 324,
+     K_OPEN = 325,
+     K_OPTION = 326,
+     K_OR = 327,
+     K_PERFORM = 328,
+     K_PRIOR = 329,
+     K_QUERY = 330,
+     K_RAISE = 331,
+     K_RELATIVE = 332,
+     K_RESULT_OID = 333,
+     K_RETURN = 334,
+     K_REVERSE = 335,
+     K_ROWTYPE = 336,
+     K_ROW_COUNT = 337,
+     K_SCROLL = 338,
+     K_SLICE = 339,
+     K_SQLSTATE = 340,
+     K_STRICT = 341,
+     K_THEN = 342,
+     K_TO = 343,
+     K_TYPE = 344,
+     K_USE_COLUMN = 345,
+     K_USE_VARIABLE = 346,
+     K_USING = 347,
+     K_VARIABLE_CONFLICT = 348,
+     K_WARNING = 349,
+     K_WHEN = 350,
+     K_WHILE = 351
+   };
 #endif
 
-/* Value type.  */
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE YYSTYPE;
-union YYSTYPE
+typedef union YYSTYPE
 {
-#line 115 "gram.y" /* yacc.c:1909  */
+
+/* Line 2068 of yacc.c  */
+#line 115 "gram.y"
 
 		core_YYSTYPE			core_yystype;
 		/* these fields must match core_YYSTYPE: */
@@ -199,29 +193,30 @@ union YYSTYPE
 		PLpgSQL_stmt_fetch		*fetch;
 		PLpgSQL_case_when		*casewhen;
 
-#line 203 "pl_gram.h" /* yacc.c:1909  */
-};
+
+
+/* Line 2068 of yacc.c  */
+#line 200 "pl_gram.h"
+} YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
-/* Location type.  */
+extern YYSTYPE plpgsql_yylval;
+
 #if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
-typedef struct YYLTYPE YYLTYPE;
-struct YYLTYPE
+typedef struct YYLTYPE
 {
   int first_line;
   int first_column;
   int last_line;
   int last_column;
-};
+} YYLTYPE;
+# define yyltype YYLTYPE /* obsolescent; will be withdrawn */
 # define YYLTYPE_IS_DECLARED 1
 # define YYLTYPE_IS_TRIVIAL 1
 #endif
 
-
-extern YYSTYPE plpgsql_yylval;
 extern YYLTYPE plpgsql_yylloc;
-int plpgsql_yyparse (void);
 
-#endif /* !YY_PLPGSQL_YY_PL_GRAM_H_INCLUDED  */
